/**************************************************************

   Program Title:  Wasp AVR Monitor
   
   Program Name:  Wasp_Utilities.c	Version: 0.75
   
   Author:  SS	  					  Date:  August 10, 2008
   
   Compiled with ImageCraft ICCAVR Standard V7.13A.
   
   This file contains routines that implement a set of basic utility 
   functions for the Wasp Monitor.
   
   Copyright 2004-2009, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2009, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: Wasp_Utilities.c,v $
 * Revision 0.86  2006/02/07  SS
 *    2-wire interface, 8MHz clock, 19,200baud
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 *
 */
#include <iom644v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "Wasp_Uart.h"
#include "Wasp_Defines.h"
#include "Wasp_Extern.h"

//*********************************************************
//*********************************************************
//      Utilities
//*********************************************************
//*********************************************************
void Delay(void)
{
	unsigned char a, b;

	for (a = 1; a; a++)
		for (b = 1; b; b++)
			;
}

void shortdelay(int cnt)
{
 	unsigned char inbyte;
	int i;
	
    for(i=0; i<cnt; i++)  {
	    inbyte = 0x55;
		}
}

void LongDelay(int cnt)
{
 	 while(cnt > 0) {
	 	 Delay();
		 cnt--;
		 }
}

void Delaymsec(int delayvalue)
{
 	 mseccountdown2 = delayvalue;
	 while(mseccountdown2) ;
}

void RED_LED(unsigned char mode)
{
 	if(mode==ON) ClrBit(PORTD,PIND5);	// low output to turn LED on
	else SetBit(PORTD,PIND5);	// low output to turn LED off
}

void GREEN_LED(unsigned char mode)
{
 	 if(mode==ON) ClrBit(PORTD,PIND4);	
	 else SetBit(PORTD,PIND4);	
}

// Function to initiate an EEPROM write
// writes the specified data byte to the specified location
// this will fail if the EEPROM is not ready!
void RTEEPROMwrite(int location, unsigned char databyte)
{
	unsigned char savedSREG;
	EEAR = location;								// set address
	EEDR = databyte;								// set data	
	savedSREG = SREG;								// keep setting so it can be restored
	CLI();													// disable interrupts
	EECR |= BIT(EEMWE);							// set "write enable" bit
	EECR |= BIT(EEWE);							// set "write" bit
	SREG = savedSREG;								// restore SREG
	EEAR = 0;
}

// function to test if the EEPROM is ready for a read or write operation
// returns non zero if ready, zero if not ready 
unsigned char RTEEPROMReady(void)
{
	return !(EECR & 0x02);
}

// EEPROM writes take about 8ms - so poll until done
// Do not use ICC V7 function EEPROMwrite() - this routine turns interrupts off!
void EEPROMWrite(int location, unsigned char databyte)
{
 	 while(!RTEEPROMReady())  ;
 	 RTEEPROMwrite(location,databyte);
 	 while(!RTEEPROMReady())  ;
}

// Write a string of characters to EEPROM at specified start location
void WriteEEPROM(int location, unsigned char *strg)
{
 	 int i;
	 
	 i = 0;
	 while(strg[i] != 0x00)  {
	     EEPROMWrite(location+i,strg[i]);
		 i++;
		 }
}

void BlinkGreen(int period)
{
 	 GREEN_LED(ON);
	 LongDelay(period);
	 GREEN_LED(OFF);
	 LongDelay(period);
}

void BlinkRed(int period)
{
 	 RED_LED(ON);
	 LongDelay(period);
	 RED_LED(OFF);
	 LongDelay(period);
}

void PrintHex(unsigned char inbyte)
{
     //printf("%2x",inbyte);
 	 bytetohex(inbyte);
	 puts(hexbyte);
}

void PrintWordHex(unsigned int inword)
{
 	 //printf("%4x",inword);
 	 addresstohex(inword);
	 puts(hexaddress);
}

//  TBD
void EEPROMSetup(void)
{
 	
}

//  Access contents of AVR flash at specified address
unsigned char ReadAVRFlash(int address)
{
 	unsigned char flashbyte,rbyte;
	
	flashbyte = 0;
 	asm("mov r30,r16 \n"
       "mov r31,r17 \n"
	   "lpm r0,Z \n"
	   "mov %flashbyte,r0 \n"); 
	rbyte = flashbyte;  
	return(rbyte);                              
}

//  Output sign on message loaded from Flash
void signon()
{
	putfs(signon1);
	putfs(signon2);
	CheckFlashTypes();
	putfs(signon3);
	putfs(signon4);
	putfs(signon5);
	putfs(signon6);
	putfs(signon62);
	putfs(signon65);
	putfs(signon66);
	putfs(signon67);
	putfs(signon7);
	putfs(signon8);
	putfs(signon83);
	putfs(signon85);
	putfs(signon9);
}


//  Display AVR EEPROM memory
void DisplayEEPROM(int address,int numbytes)
{
 	 unsigned char i,byte,bytesonlastline,asciicnt,crcvalue,tempvalue;
	 int linecnt,numlines,newaddress;

	 linecnt = asciicnt = 0;
	 bytesonline = 16;	 
	 numlines = numbytes/bytesonline;
	 bytesonlastline = numbytes - numlines*bytesonline;
	 while(linecnt<numlines) {
	     crcvalue = 0;
	     putcharr(':');
		 PrintHex(bytesonline);		 //  Output bytes on line
		 crcvalue += bytesonline;
	     //  Output hex start address
		 newaddress = address+linecnt*16;
	     PrintWordHex(newaddress);
		 tempvalue = (newaddress&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = newaddress&0x00ff;
		 crcvalue += tempvalue;
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonline; i++) {
	         byte = EEPROM_read(i+newaddress);
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         PrintHex(byte);
	         }
		 linecnt++;
		 asciicnt = 0;
		 // Output CRC
		 PrintHex((~crcvalue)+0x01);
	     puts("  ");
		 for(i=0; i<bytesonline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putcharr(ascii[i]);
			 else putcharr('.');
			 }
	     putCRLF();  
		 }
	 if(bytesonlastline) {
	     crcvalue = 0;
	     putcharr(':');
		 //  Output bytes on line
		 PrintHex(bytesonlastline);
		 crcvalue += bytesonlastline;
	     //  Output hex start address
		 newaddress = address+linecnt*16;
	     addresstohex(newaddress);
		 tempvalue = (newaddress&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = newaddress&0x00ff;
		 crcvalue += tempvalue;
	     puts(hexaddress);
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonlastline; i++) {
	         byte = EEPROM_read(i+address+linecnt*16);
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         PrintHex(byte);
	         }
		 // Output CRC
		 PrintHex((~crcvalue)+0x01);
	     puts("  ");
		 for(i=0; i<bytesonlastline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putcharr(ascii[i]);
			 else putcharr('.');
			 }
	     putCRLF();  
		 }
}

//  Display P1 AVR Flash memory
void DisplayAVRFlash(int address,int numbytes)
{
 	 unsigned char i,byte,bytesonlastline,asciicnt,crcvalue,tempvalue;
	 int linecnt,numlines,newaddress;

	 linecnt = asciicnt = 0;
	 bytesonline = 16;	 
	 numlines = numbytes/bytesonline;
	 bytesonlastline = numbytes - numlines*bytesonline;
	 while(linecnt<numlines) {
	 	 crcvalue = 0;
	     putcharr(':');
		 //  Output bytes on line
		 PrintHex(bytesonline);
		 crcvalue += bytesonline;
	     //  Output hex start address
		 newaddress = address+linecnt*16;
	     PrintWordHex(newaddress);
		 tempvalue = (newaddress&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = newaddress&0x00ff;
		 crcvalue += tempvalue;
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonline; i++) {
	         byte = ReadAVRFlash(address+i+linecnt*16);
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         PrintHex(byte);
	         }
		 linecnt++;
		 asciicnt = 0;
		 // Output CRC
	     PrintHex((~crcvalue)+0x01);
	     puts("  ");
		 for(i=0; i<bytesonline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putcharr(ascii[i]);
			 else putcharr('.');
			 }
	     putCRLF();  
		 }
	 if(bytesonlastline) {
	     crcvalue = 0;
	     putcharr(':');
		 //  Output bytes on line
		 PrintHex(bytesonlastline);
	     //  Output hex start address
	     PrintWordHex(address+linecnt*16);
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonlastline; i++) {
	         byte = ReadAVRFlash(address+i+linecnt*16);
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         PrintHex(byte);
	         }
		 // Output CRC
	     PrintHex((~crcvalue)+0x01);
	     puts("  ");
		 for(i=0; i<bytesonlastline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putcharr(ascii[i]);
			 else putcharr('.');
			 }
	     putCRLF();  
		 }
}

unsigned char hextobyte(unsigned char inbyte)
{
	 if((inbyte >= 'A')&(inbyte<='F')) return (inbyte - 'A' + 10);
	 if((inbyte >= 'a')&(inbyte<='f')) return (inbyte - 'a' + 10);
	 else return (inbyte - '0');
}

// Read in a hex string and convert to int or long
//  TBD - add long support for serial flash support
unsigned int  ScanHex()
{
 	 unsigned int  address;
 	 char i=0,len;
	 
	 address = 0;
	 putcharr((instrg[0]=getcharr()));
 	 while((instrg[i]!=CR)&&(instrg[i]!=' ')) {
	 	 i++;
	     putcharr((instrg[i]=getcharr()));
		 }
	 instrg[i] = 0;
	 len = strlen(instrg);
	 address = hextobyte(instrg[len-1]);
	 i = 1;
	 while(i<len) {
	     if(i==1) address += (hextobyte(instrg[len-i-1])*16);
	     if(i==2) address += (hextobyte(instrg[len-i-1])*256);
	     if(i==3) address += (hextobyte(instrg[len-i-1])*4096);
		 i++;
		 }
	 return address;
}

unsigned char GetByteHex()
{
 	 char strg[3];
	 
//	 putcharr((strg[0]=getchar()));
//	 putcharr((strg[1]=getchar()));
	 strg[0]=getcharr();
	 strg[1]=getcharr();
	 strg[2] = 0;
	 return converttohex(strg);
}

void DisplayMemory(void)
{
    unsigned char inchar;
	unsigned int address, length;
	
	inchar = getcharr();
	putcharr(inchar);
	putCRLF();
	putfs(displaymessage1);
	address = ScanHex();
	length = ScanHex();
	putCRLF();
	switch (inchar) {
	    case 'e': case 'E':
			DisplayEEPROM(address,length);
		    break;
		case 'a': case 'A':
		    DisplayAVRFlash(address,length);
		    break;
		//  In order to access Blackfin Flash the AVR must ask the Blackfin
		//  to release control of the SPI lines.  This is simulated in Rev A
		//  boards by resetting the Blackfin (which should release the SPI lines)
		//  On completion the AVR SPI lines are released.
		case 'f': case 'F':
			Init_SPI_Master();
		    DisplaySerialFlash(address,length,READCONTINUOUS);	
			Release_SPI();
		    break;
		case 'b': case 'B':
			Init_SPI_Master();
		    DisplaySerialFlash(address,length,READBUFFER);			
			Release_SPI();
		    break;
		default:
		    putfs(displaymessage33);
		    break;
	    }
}


// Check intelbinary CRC 
int checkcrc(unsigned char cycbyte)
{
	GetByteHex();
 	return 1;
}

// Convert Intel Hex ASCII string to binary
//
// :10 0100 00 0C948F0D0C941D0E0C941D0E0C941D0E 38
// :1001000000C941D0E0C941D0E0C944E590C941D0E38
// This routine decodes a line of Intel Hex.  Other 
// routines write the data to either AVR Flash, external Flash
// or AVR EEPROM
//
//   :02 0000 04 0000FA  - IntelHex32 super address record
// 
//  TBD - check crc byte - drop record if incorrect
int DecodeIntelHex(void)
{
 	 unsigned char strg[8],addressbyte,inbyte;
	 int i;
	 
	 putcharr(':');
	 // Convert count of bytes on line
	 bytesonline = GetByteHex();
	 
	 // Convert address to binary - ignore filler byte
	 // Typical Intel Hex consists of 4 ASCII characters
	 inteladdress = GetByteHex()*256;
	 inteladdress += GetByteHex();
	 
	 // Read dump byte (2 characters)
	 intelrecordcommand = GetByteHex();
	 
	 if(intelrecordcommand==0x04) {
	     intelsuperaddress = GetByteHex()*256;
		 intelsuperaddress += GetByteHex();
		 }
	 else  {
	     // Convert all data to binary
	     for(i=0; i<bytesonline; i++) {
		     inbyte = GetByteHex(strg);
		     intelbinary[i] = inbyte;
	 	     }
		 }
		 
	// Check CRC - todo
	if(checkcrc(inbyte)) return 1;
	else return -1;
}

//  Write decoded Intel hex to EEPROM
void ProgramEEPROM(void)
{
     unsigned char i;
	 
	 for(i=0; i<bytesonline; i++)  EEPROMWrite(inteladdress+i,intelbinary[i]);
}

//  TBD - AVR Flash hold program so programming AVR flash may overwrite program
void ProgramAVRFlash(void)
{
 	 char inchar;
	 
 	 while((inchar=getcharr())!=CR) putcharr(inchar);
	 printf("\n-");
}

void CleanUpFlashState()
{
	if(flashdatainbuffer==YES)  {
	    inteladdress = oldinteladdress;  // end of file record destroyed intelad
	    ProgramBuffertoPageErase(1);
		WaitForFlashFree();
		}
	if(flashprogrammingactive==YES)  {
		Release_SPI();
		flashprogrammingactive = NO;
		flashpageloaded = NO;
		flashpageboundarytraversed = NO;
		flashdatainbuffer = NO;
		}
}

//  Main memory programming selection function
//  Program EEPROM, AVR Flash or SPI Flash.
//  Three IntelHex32 Records 0x00, 0x01 and 0x04 are processed by this
//  routine.  0x00 is a data record, 0x01 is End of File record and 
//  0x04 is extended address record
//
//  TBD - no intel record time out - reset defaults if not intel 
//        records received after 3 seconds
void ProgramMemory(void)
{
 	DecodeIntelHex();
	switch (programmode) {
	    case LEEPROM:
			if(intelrecordcommand==0) ProgramEEPROM();
			break;
		case LAVRFLASH:
		    if(intelrecordcommand==0) ProgramAVRFlash();
			break;
			
		//  To access Blackfin Flash the Blackfin must not have control 
		//  of the shared SPI lines and the Blackfin reset line must not
		//  be active.  The Blackfin releases control of the SPI lines under 
		//  software control or by being forced to boot mode 0.  If the Blackfin
		//  is reset to boot mode 3 it will attempt to load a program from 
		//  serial flash preventing the AVR from accessing the serial flash.
		
		//  This implementation forces the Blackfin to reboot.
		//  Cooperative flash access still TBD
		case LFLASH:
		    // Data record			
			if(intelrecordcommand==0) {
			    flashcountdown = 3000;  // 3 second no new Hex record timeout
			    //  See is SPI Master is active - set active if not
			    if(flashprogrammingactive==NO) {
				    Init_SPI_Master();
					}
					
		        ProgramSerialFlash();		
				}
			// Extended address record
			else if(intelrecordcommand==0x04) ; // nothing to do
			
			// End of File record - reset all variables
			else if(intelrecordcommand==0x01) CleanUpFlashState();
			break;
		} 
}

//  Set default memory programming mode
//  Subsequent Intel Hex dumps will load to the specific memory
void LoadMemory(void)
{
    unsigned char inchar;
	
	inchar = getcharr();
	putcharr(inchar);
	switch (inchar) {
	    case 'e': case 'E':
			programmode = LEEPROM;
			putfs(loadmessage1);
		    break;
		case 'f': case 'F':
			programmode = LFLASH;
			putfs(loadmessage2);
		    break;
		case 'a': case 'A':
			programmode = LAVRFLASH;
		    putfs(loadmessage3);
		    break;
		default:
		   break;
	    }
}

//  Output program version
//   TBD - read from EEPROM
void ProgramVersion(void)
{
      char version[5]="0.91";
	  
	  putfs(progmessage);
	  puts(version);
	  putCRLF();
}

//  Read selected analog channel - wait until it completes
//   TBD - background interrupt process
int AnalogCh(int ch,char mode)
{
 	unsigned char low,high;
	
	if(ch==0) ADMUX = 0x00;  // Select AVCC, ch 0
	else if(ch==1) ADMUX = 0x01; // 
	else if(ch==2) ADMUX = 0x02;
	else if(ch==3) ADMUX = 0x03;
	else if(ch==4) ADMUX = 0x04; 
	else if(ch==5) ADMUX = 0x05;
	else if(ch==6) ADMUX = 0x06;
	else if(ch==7) ADMUX = 0x07;
	else if(ch==8) ADMUX = 0x0C;  // Boost ADC2 gain by 10x
	if(mode==INTERNALREF)  ADMUX |= 0xC0;
	else ADMUX |= 0x40;  // use AVCC
	
	ADCSRA = (1<<ADEN)|(1<<ADSC); // Enable ADC, start conversion, max clock
	while(!(ADCSRA&(1<<ADIF))) ; // loop until conversion completes
	shortdelay(10);
	
	low = ADCL;
	high = ADCH;
	return (low + (high*256));
}

void DumpAnalogChs(void)
{
    int  i,analogvalue;
	
	printf("Analog Channels 0..7\n");
 	for(i=0; i<8; i++) {
	    analogvalue = AnalogCh(i,EXTERNALREF);
		printf("%d ",analogvalue);
		} 
	printf("\n"); 
}

// Collect characters until CR, LF or space found
int readint()
{
 	char incharbuf[10],inchar;
	int  intnum,i,endptr;
	
	intnum = 0;
	i = 0;
	// Skip preceeding white space
	while((inchar=getcharr())==' ') putchar(inchar);
	putchar(inchar);
	while((i<10)&&(inchar!=CR)&&(inchar!=LF)&&(inchar!=' ')) {
	    incharbuf[i++] = inchar;
		inchar = getcharr();
		putchar(inchar);
		}
	incharbuf[i] = 0;
	endptr = strlen(incharbuf) - 1;
	if(endptr>=0) intnum = (incharbuf[endptr--] - '0');
	if(endptr>=0) intnum += (incharbuf[endptr--] - '0')*10;
	if(endptr>=0) intnum += (incharbuf[endptr--] - '0')*100;
	if(endptr>=0) intnum += (incharbuf[endptr--] - '0')*1000;
	if(endptr>=0) intnum = (incharbuf[endptr--] - '0')*10000;	
	return intnum;
}

char getint()
{
	char intvalue,inchar;
	
	inchar = getcharr();
	putcharr(inchar);
	if(inchar==CR) return 0x78;
	intvalue = (inchar - '0')*10;
	inchar = getcharr();
	putcharr(inchar);	
	intvalue = intvalue + (inchar-'0');
	return intvalue;
}

// Update the current day and time
void  getsettime()
{
    unsigned char instrg[4],inchar;
	
    _CLI();
	instrg[0] = days;
	instrg[1] = hours;
	instrg[2] = minutes;
	instrg[3] = seconds;
	_SEI();	
	
	printf("Set time [hh:mm:ss]=%d %d:%d:%d :",
	   		instrg[0],instrg[1],instrg[2],instrg[3]);

	if((instrg[1]=getint())==0x78) return;
	inchar = getcharr(); putcharr(inchar);
	instrg[2] = getint();
	inchar = getcharr(); putcharr(inchar);
	instrg[3] = getint();
	inchar = getcharr(); putcharr(inchar);

    _CLI();
	days = 0;
	hours = instrg[1];
	minutes = instrg[2];
	seconds = instrg[3];
	_SEI();	
}

//  Simple test routine
void TestAVR(void)
{
 	 char i,j;
	 
 	 printf("\nTesting AVR Version 0.34\n");
//	 TestAVRRAM();
//	 TestAVREEPROM();
//	 TestAVRFLASH();
//	 TestBoardFLASH():
	 j = 0;
	 putcharr(' ');
	 for(i=0x20; i<0x7f; i++) {
	 	 putcharr(i);
		 if(j++ == 60)  {
		     j = 0;
			 printf("\n ");
			 }
		 }
	 printf("\n ");
	 DumpAnalogChs();
}

void  EEPROM_write(int uiAddress,unsigned char ucData)
{
    //  Wait for completion of previous write
	while(EECR&(1<<EEPE));
	
	// Set up address and Data registers
	EEAR = uiAddress;
	EEDR = ucData;
	
	// Write logical one to EEMPE
	EECR |= (1<<EEMPE);
	
	// Start eeprom write by setting EEPE
	EECR |= (1<<EEPE);
}

unsigned char EEPROM_read(int uiAddress)
{
    // Wait for completion of previous write
	while(EECR&(1<<EEPE));
	
	// Setup address
	EEAR = uiAddress;
	
	// Start eeprom read by writing EERE
	EECR |= (1<<EERE);
	
	// Return data from data register
	return EEDR;
}

//  Signature bytes are write to first five EEPROM locations 85 C2 1E 4F 11
void WriteEepromSignature(void)
{
    EEPROM_write(0,0x85);
    EEPROM_write(1,0xC2);
    EEPROM_write(2,0x1E);
    EEPROM_write(3,0x4F);
    EEPROM_write(4,0x11);
}

void WriteEepromInt(int address,int ivalue)
{
    union u_tag {
	    int iivalue;
		unsigned char intchar[2];
		} u;
	
	u.iivalue = ivalue;
 	EEPROM_write(address++,u.intchar[0]);
 	EEPROM_write(address,u.intchar[1]);	
}

// Write program version - offset address 5
void WriteVersion(void)
{
    EEPROM_write(5,PRODUCTID);
	EEPROM_write(6,PROGRAMMAJORVER);
	EEPROM_write(7,PROGRAMMINORVER);
}

// Write copyright notice - offset address 8
void WriteCopyright(void)
{
    unsigned char ptr;
	
	for(ptr=8; ptr<(8+strlen(version)); ptr++)
          EEPROM_write(ptr,version[ptr-8]);
}

// Check 5 signature bytes in first five locations of EEPROM
//   Should be 85 C2 1E 4F 11
unsigned char CheckEepromSignature(void)
{
 	unsigned char status;
	
	status = 1;
	if(EEPROM_read(0)!=0x85) status = 0;
	if(EEPROM_read(1)!=0xC2) status = 0;
	if(EEPROM_read(2)!=0x1E) status = 0;
	if(EEPROM_read(3)!=0x4F) status = 0;
	if(EEPROM_read(4)!=0x11) status = 0;
	return status;
}

int ReadEepromInt(int address)
{	
    union u_tag {
	    int ivalue;
		unsigned char intchar[2];
		} u;
	
	u.intchar[0] = EEPROM_read(address++);
	u.intchar[1] = EEPROM_read(address);
	return u.ivalue;	
}

void  WriteEepromFloat(int address,float fvalue)
{
    union u_tag {
	    float ffvalue;
		unsigned char floatchar[4];
		} u;
	
	u.ffvalue = fvalue;
 	EEPROM_write(address++,u.floatchar[0]);
 	EEPROM_write(address++,u.floatchar[1]);
 	EEPROM_write(address++,u.floatchar[2]);
 	EEPROM_write(address,u.floatchar[3]);	
}

float ReadEepromFloat(int address)
{	
    union u_tag {
	    float fvalue;
		unsigned char floatchar[4];
		} u;
	
	u.floatchar[0] = EEPROM_read(address++);
	u.floatchar[1] = EEPROM_read(address++);
	u.floatchar[2] = EEPROM_read(address++);
	u.floatchar[3] = EEPROM_read(address);
	return u.fvalue;	
}

//  Load all the Motor Controller Setup preferences from EEPROM
void LoadEepromPreferences()
{
    unsigned char inchar;
    int addressptr;
	
	addressptr = 33;
	
	// I2C Configuration
	i2cmode = EEPROM_read(addressptr++);
	TWI_slaveAddress = EEPROM_read(addressptr);
	TWI_masterAddress = EEPROM_read(addressptr++);
	
	// General Setup parameters
	defaultlograte = ReadEepromInt(addressptr);  addressptr += 2;
	armpd2 = EEPROM_read(addressptr++);
	data2log = EEPROM_read(addressptr++);
	
/*	followstepdir = EEPROM_read(addressptr++);

	
	// Smart command mode configuration
	recognizecommandmode = EEPROM_read(addressptr++);
	
	// Motor drive parameters
	resolution = ReadEepromFloat(addressptr); addressptr += 4;
	minpps = ReadEepromFloat(addressptr); addressptr += 4;
	maxpps = ReadEepromFloat(addressptr); addressptr += 4;
	ramprate = ReadEepromFloat(addressptr); addressptr += 4;
	feedrate = ReadEepromFloat(addressptr); addressptr += 4;
	feedratemax = ReadEepromFloat(addressptr); addressptr += 4;
	backlash = ReadEepromInt(addressptr);  addressptr += 2;*/
}

// Save controller configuration information to EEPROM
void SaveEepromPreferences()
{
    int addressptr;

	addressptr = 33;

	// I2C Configuration
	EEPROM_write(addressptr++,i2cmode);
//	EEPROM_write(addressptr++,TWI_slaveAddress);
	EEPROM_write(addressptr++,TWI_masterAddress);

	// General Setup parameters
	WriteEepromInt(addressptr,defaultlograte); addressptr += 2;
	EEPROM_write(addressptr++,armpd2);
	EEPROM_write(addressptr++,data2log);

	// Smart command mode configuration
/*	EEPROM_write(addressptr++,recognizecommandmode);
	
	// Motor drive parameters
	WriteEepromFloat(addressptr,resolution); addressptr += 4;

	WriteEepromInt(addressptr,backlash); addressptr += 2;*/
}
